@@ -47,6 +47,7 @@ import org.apache.qpid.jms.test.testpeer.TestAmqpPeer;
 import org.apache.qpid.jms.transports.TransportOptions;
 import org.apache.qpid.jms.transports.TransportSupport;
 import org.apache.qpid.proton.amqp.Symbol;
+import org.apache.qpid.proton.amqp.UnsignedByte;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -60,11 +61,17 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     private static final Symbol CRAM_MD5 = Symbol.valueOf("CRAM-MD5");
     private static final Symbol SCRAM_SHA_1 = Symbol.valueOf("SCRAM-SHA-1");
     private static final Symbol SCRAM_SHA_256 = Symbol.valueOf("SCRAM-SHA-256");
+    private static final Symbol SCRAM_SHA_512 = Symbol.valueOf("SCRAM-SHA-512");
     private static final Symbol EXTERNAL = Symbol.valueOf("EXTERNAL");
     private static final Symbol XOAUTH2 = Symbol.valueOf("XOAUTH2");
 
-    private static final String BROKER_JKS_KEYSTORE = "src/test/resources/broker-jks.keystore";
-    private static final String BROKER_JKS_TRUSTSTORE = "src/test/resources/broker-jks.truststore";
+    private static final UnsignedByte SASL_FAIL_AUTH = UnsignedByte.valueOf((byte) 1);
+    private static final UnsignedByte SASL_SYS = UnsignedByte.valueOf((byte) 2);
+    private static final UnsignedByte SASL_SYS_PERM = UnsignedByte.valueOf((byte) 3);
+    private static final UnsignedByte SASL_SYS_TEMP = UnsignedByte.valueOf((byte) 4);
+
+    private static final String BROKER_PKCS12_KEYSTORE = "src/test/resources/broker-pkcs12.keystore";
+    private static final String BROKER_PKCS12_TRUSTSTORE = "src/test/resources/broker-pkcs12.truststore";
     private static final String CLIENT_JKS_KEYSTORE = "src/test/resources/client-jks.keystore";
     private static final String CLIENT_JKS_TRUSTSTORE = "src/test/resources/client-jks.truststore";
     private static final String PASSWORD = "password";
@@ -72,10 +79,10 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     @Test(timeout = 20000)
     public void testSaslExternalConnection() throws Exception {
         TransportOptions sslOptions = new TransportOptions();
-        sslOptions.setKeyStoreLocation(BROKER_JKS_KEYSTORE);
+        sslOptions.setKeyStoreLocation(BROKER_PKCS12_KEYSTORE);
         sslOptions.setKeyStorePassword(PASSWORD);
         sslOptions.setVerifyHost(false);
-        sslOptions.setTrustStoreLocation(BROKER_JKS_TRUSTSTORE);
+        sslOptions.setTrustStoreLocation(BROKER_PKCS12_TRUSTSTORE);
         sslOptions.setTrustStorePassword(PASSWORD);
 
         String connOptions = "?transport.trustStoreLocation=" + CLIENT_JKS_TRUSTSTORE + "&" +
@@ -225,6 +232,32 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
         }
     }
 
+    @Test(timeout = 20000)
+    public void testSaslFailureCodes() throws Exception {
+        doSaslFailureCodesTestImpl(SASL_FAIL_AUTH);
+        doSaslFailureCodesTestImpl(SASL_SYS);
+        doSaslFailureCodesTestImpl(SASL_SYS_PERM);
+        doSaslFailureCodesTestImpl(SASL_SYS_TEMP);
+    }
+
+    private void doSaslFailureCodesTestImpl(UnsignedByte saslFailureCode) throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+
+            testPeer.expectSaslFailingExchange(new Symbol[] {PLAIN, ANONYMOUS}, PLAIN, saslFailureCode);
+
+            ConnectionFactory factory = new JmsConnectionFactory("amqp://localhost:" + testPeer.getServerPort() + "?jms.clientID=myClientID");
+
+            try {
+                factory.createConnection("username", "password");
+                fail("Excepted exception to be thrown");
+            }catch (JMSSecurityException jmsse) {
+                LOG.info("Caught expected security exception: {}", jmsse.getMessage());
+            }
+
+            testPeer.waitForAllHandlersToComplete(1000);
+        }
+    }
+
     /**
      * Add a small delay after the SASL process fails, test peer will throw if
      * any unexpected frames arrive, such as erroneous open+close.
@@ -261,6 +294,11 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
         doMechanismSelectedTestImpl("username", "password", SCRAM_SHA_256, new Symbol[] {SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS}, false);
     }
 
+    @Test(timeout = 20000)
+    public void testScramSha512SelectedWhenCredentialsPresent() throws Exception {
+        doMechanismSelectedTestImpl("username", "password", SCRAM_SHA_512, new Symbol[] {SCRAM_SHA_512, SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS}, false);
+    }
+
     @Test(timeout = 20000)
     public void testXoauth2SelectedWhenCredentialsPresent() throws Exception {
         String token = Base64.getEncoder().encodeToString("token".getBytes(StandardCharsets.US_ASCII));
@@ -270,7 +308,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     private void doMechanismSelectedTestImpl(String username, String password, Symbol clientSelectedMech, Symbol[] serverMechs, boolean wait) throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
 
-            testPeer.expectFailingSaslAuthentication(serverMechs, clientSelectedMech);
+            testPeer.expectSaslFailingAuthentication(serverMechs, clientSelectedMech);
 
             ConnectionFactory factory = new JmsConnectionFactory("amqp://localhost:" + testPeer.getServerPort() + "?jms.clientID=myclientid");
             try {
@@ -294,21 +332,21 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
 
     @Test(timeout = 20000)
     public void testExternalSelectedWhenLocalPrincipalPresent() throws Exception {
-        doMechanismSelectedExternalTestImpl(true, EXTERNAL, new Symbol[] {EXTERNAL, SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS});
+        doMechanismSelectedExternalTestImpl(true, EXTERNAL, new Symbol[] {EXTERNAL, SCRAM_SHA_512, SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS});
     }
 
     @Test(timeout = 20000)
     public void testExternalNotSelectedWhenLocalPrincipalMissing() throws Exception {
-        doMechanismSelectedExternalTestImpl(false, ANONYMOUS, new Symbol[] {EXTERNAL, SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS});
+        doMechanismSelectedExternalTestImpl(false, ANONYMOUS, new Symbol[] {EXTERNAL, SCRAM_SHA_512, SCRAM_SHA_256, SCRAM_SHA_1, CRAM_MD5, PLAIN, ANONYMOUS});
     }
 
     private void doMechanismSelectedExternalTestImpl(boolean requireClientCert, Symbol clientSelectedMech, Symbol[] serverMechs) throws Exception {
         TransportOptions sslOptions = new TransportOptions();
-        sslOptions.setKeyStoreLocation(BROKER_JKS_KEYSTORE);
+        sslOptions.setKeyStoreLocation(BROKER_PKCS12_KEYSTORE);
         sslOptions.setKeyStorePassword(PASSWORD);
         sslOptions.setVerifyHost(false);
         if (requireClientCert) {
-            sslOptions.setTrustStoreLocation(BROKER_JKS_TRUSTSTORE);
+            sslOptions.setTrustStoreLocation(BROKER_PKCS12_TRUSTSTORE);
             sslOptions.setTrustStorePassword(PASSWORD);
         }
 
@@ -323,7 +361,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
                                "transport.keyStorePassword=" + PASSWORD;
             }
 
-            testPeer.expectFailingSaslAuthentication(serverMechs, clientSelectedMech);
+            testPeer.expectSaslFailingAuthentication(serverMechs, clientSelectedMech);
 
             JmsConnectionFactory factory = new JmsConnectionFactory("amqps://localhost:" + testPeer.getServerPort() + connOptions);
             try {
@@ -385,7 +423,7 @@ public class SaslIntegrationTest extends QpidJmsTestCase {
     private void doMechanismSelectionRestrictedTestImpl(String username, String password, Symbol clientSelectedMech, Symbol[] serverMechs, String mechanismsOptionValue) throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
 
-            testPeer.expectFailingSaslAuthentication(serverMechs, clientSelectedMech);
+            testPeer.expectSaslFailingAuthentication(serverMechs, clientSelectedMech);
 
             String uriOptions = "?jms.clientID=myclientid";
             if(mechanismsOptionValue != null) {
