@@ -23,7 +23,6 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
-import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.concurrent.TimeUnit;
@@ -37,6 +36,7 @@ import org.apache.qpid.jms.meta.JmsResource;
 import org.apache.qpid.jms.meta.JmsResourceId;
 import org.apache.qpid.jms.meta.JmsResourceVistor;
 import org.apache.qpid.jms.provider.DefaultProviderListener;
+import org.apache.qpid.jms.provider.ProviderException;
 import org.apache.qpid.jms.provider.ProviderFuture;
 import org.apache.qpid.jms.test.QpidJmsTestCase;
 import org.apache.qpid.jms.test.testpeer.TestAmqpPeer;
@@ -161,11 +161,12 @@ public class AmqpProviderTest extends QpidJmsTestCase {
         try (TestAmqpPeer testPeer = new TestAmqpPeer()) {
 
             provider = new AmqpProviderFactory().createProvider(getPeerURI(testPeer));
-
             provider.setSaslLayer(false);
-            provider.connect(connectionInfo);
 
             testPeer.expectSaslLayerDisabledConnect(null);
+
+            provider.connect(connectionInfo);
+
             testPeer.expectClose();
 
             provider.close();
@@ -316,18 +317,18 @@ public class AmqpProviderTest extends QpidJmsTestCase {
             try {
                 provider.start();
                 fail("Should have thrown an IOException when closed.");
-            } catch (IOException ex) {}
+            } catch (Exception ex) {}
 
             try {
                 provider.connect(connectionInfo);
                 fail("Should have thrown an IOException when closed.");
-            } catch (IOException ex) {}
+            } catch (Exception ex) {}
 
             ProviderFuture request = provider.newProviderFuture();
             try {
                 provider.unsubscribe("subscription-name", request);
                 fail("Should have thrown an IOException when closed.");
-            } catch (IOException ex) {}
+            } catch (Exception ex) {}
 
             provider.close();
 
@@ -377,27 +378,32 @@ public class AmqpProviderTest extends QpidJmsTestCase {
 
     @Test(timeout = 20000)
     public void testErrorDuringCreateResourceFailsRequest() throws Exception {
-        doErrorDuringOperationFailsRequesTTestImpl(Op.CREATE);
+        doErrorDuringOperationFailsRequestTestImpl(Op.CREATE);
     }
 
     @Test(timeout = 20000)
     public void testErrorDuringStartResourceFailsRequest() throws Exception {
-        doErrorDuringOperationFailsRequesTTestImpl(Op.START);
+        doErrorDuringOperationFailsRequestTestImpl(Op.START);
     }
 
     @Test(timeout = 20000)
     public void testErrorDuringStopResourceFailsRequest() throws Exception {
-        doErrorDuringOperationFailsRequesTTestImpl(Op.STOP);
+        doErrorDuringOperationFailsRequestTestImpl(Op.STOP);
     }
 
     @Test(timeout = 20000)
     public void testErrorDuringDestroyResourceFailsRequest() throws Exception {
-        doErrorDuringOperationFailsRequesTTestImpl(Op.DESTROY);
+        doErrorDuringOperationFailsRequestTestImpl(Op.DESTROY);
     }
 
-    private void doErrorDuringOperationFailsRequesTTestImpl(Op operation) throws Exception {
+    private void doErrorDuringOperationFailsRequestTestImpl(Op operation) throws Exception {
         try (TestAmqpPeer testPeer = new TestAmqpPeer()) {
+            testPeer.expectSaslAnonymous();
+            testPeer.expectOpen();
+            testPeer.expectClose();
+
             provider = new AmqpProviderFactory().createProvider(getPeerURI(testPeer));
+            provider.connect(connectionInfo);
 
             final AtomicBoolean errorThrown = new AtomicBoolean();
             JmsResource resourceInfo = new JmsAbstractResource() {
@@ -437,7 +443,7 @@ public class AmqpProviderTest extends QpidJmsTestCase {
             try {
                 request.sync();
                 fail("Request should have failed");
-            } catch (IOException e) {
+            } catch (Exception e) {
                 // Expected
             }
 
@@ -449,6 +455,94 @@ public class AmqpProviderTest extends QpidJmsTestCase {
         }
     }
 
+    @Test(timeout = 20000)
+    public void testCreateResourceFailsWhenNoConnectCalled() throws Exception {
+        doErrorDuringOperationFailsWhenNoConnectCalledTestImpl(Op.CREATE);
+    }
+
+    @Test(timeout = 20000)
+    public void testStartResourceFailsWhenNoConnectCalled() throws Exception {
+        doErrorDuringOperationFailsWhenNoConnectCalledTestImpl(Op.START);
+    }
+
+    @Test(timeout = 20000)
+    public void testStopResourceFailsWhenNoConnectCalled() throws Exception {
+        doErrorDuringOperationFailsWhenNoConnectCalledTestImpl(Op.STOP);
+    }
+
+    @Test(timeout = 20000)
+    public void testDestroyResourceFailsWhenNoConnectCalled() throws Exception {
+        doErrorDuringOperationFailsWhenNoConnectCalledTestImpl(Op.DESTROY);
+    }
+
+    private void doErrorDuringOperationFailsWhenNoConnectCalledTestImpl(Op operation) throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer()) {
+
+            provider = new AmqpProviderFactory().createProvider(getPeerURI(testPeer));
+
+            final AtomicBoolean errorThrown = new AtomicBoolean();
+            JmsResource resourceInfo = new JmsAbstractResource() {
+                @Override
+                public void visit(JmsResourceVistor visitor) {
+                    errorThrown.set(true);
+                    throw new Error("Deliberate error for testing");
+                }
+
+                @Override
+                public JmsResourceId getId() {
+                    return new JmsAbstractResourceId() {
+                    };
+                }
+            };
+
+            assertFalse("Error should not have been thrown", errorThrown.get());
+            ProviderFuture request = provider.newProviderFuture();
+
+            switch(operation) {
+            case CREATE:
+                try {
+                    provider.create(resourceInfo, request);
+                    fail("Request should have failed");
+                } catch (ProviderException e) {
+                    // Expected
+                }
+                break;
+            case START:
+                try {
+                    provider.start(resourceInfo, request);
+                    fail("Request should have failed");
+                } catch (ProviderException e) {
+                    // Expected
+                }
+                break;
+            case STOP:
+                try {
+                    provider.stop(resourceInfo, request);
+                    fail("Request should have failed");
+                } catch (ProviderException e) {
+                    // Expected
+                }
+                break;
+            case DESTROY:
+                try {
+                    provider.destroy(resourceInfo, request);
+                    fail("Request should have failed");
+                } catch (ProviderException e) {
+                    // Expected
+                }
+                break;
+            default:
+                throw new IllegalArgumentException("Unexpected operation given");
+            }
+
+            assertFalse("Error should not have been thrown", errorThrown.get());
+
+            provider.close();
+
+            testPeer.waitForAllHandlersToComplete(1000);
+        }
+    }
+
     private JmsConnectionInfo createConnectionInfo() {
         JmsConnectionId connectionId = new JmsConnectionId(connectionIdGenerator.generateId());
         JmsConnectionInfo connectionInfo = new JmsConnectionInfo(connectionId);
