@@ -16,22 +16,22 @@
  */
 package org.apache.qpid.jms.transports.netty;
 
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.collection.IsIterableContainingInAnyOrder.containsInAnyOrder;
 import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 
 import java.net.URI;
+import java.net.URLEncoder;
 import java.util.Arrays;
 import java.util.List;
 
 import org.apache.qpid.jms.transports.Transport;
 import org.apache.qpid.jms.transports.TransportOptions;
-import org.apache.qpid.jms.transports.TransportSslOptions;
 import org.junit.Test;
 
 /**
@@ -47,6 +47,9 @@ public class NettySslTransportFactoryTest {
     public static final int CUSTOM_SO_LINGER = Short.MIN_VALUE;
     public static final int CUSTOM_SO_TIMEOUT = 10;
     public static final int CUSTOM_CONNECT_TIMEOUT = 90000;
+    private static final String CUSTOM_LOCAL_ADDRESS = "localhost";
+    private static final int CUSTOM_LOCAL_PORT = 30000;
+
     public static final String CUSTOM_CONTEXT_PROTOCOL = "TLSv1.2";
     public static final String[] CUSTOM_ENABLED_PROTOCOLS = { "TLSv1.1", "TLSv1.2" };
     public static final String CUSTOM_ENABLED_PROTOCOLS_STRING = "TLSv1.1,TLSv1.2";
@@ -61,6 +64,8 @@ public class NettySslTransportFactoryTest {
     public static final boolean CUSTOM_TRUST_ALL = true;
     public static final boolean CUSTOM_VERIFY_HOST = false;
     public static final String CUSTOM_KEY_ALIAS = "myTestAlias";
+    public static final String CUSTOM_TRUST_STORE_LOCATION = "/home/user/store";
+    public static final String CUSTOM_TRUST_STORE_PASSWORD = "pass+word";
 
     @Test
     public void testCreateWithDefaultOptions() throws Exception {
@@ -86,19 +91,18 @@ public class NettySslTransportFactoryTest {
         assertEquals(TransportOptions.DEFAULT_TCP_KEEP_ALIVE, options.isTcpKeepAlive());
         assertEquals(TransportOptions.DEFAULT_SO_LINGER, options.getSoLinger());
         assertEquals(TransportOptions.DEFAULT_SO_TIMEOUT, options.getSoTimeout());
+        assertNull(options.getLocalAddress());
+        assertEquals(TransportOptions.DEFAULT_LOCAL_PORT, options.getLocalPort());
 
-        assertTrue(options instanceof TransportSslOptions);
-        TransportSslOptions sslOptions = (TransportSslOptions) options;
+        assertEquals(TransportOptions.DEFAULT_CONTEXT_PROTOCOL, options.getContextProtocol());
+        assertNull(options.getEnabledProtocols());
+        assertArrayEquals(TransportOptions.DEFAULT_DISABLED_PROTOCOLS.toArray(new String[0]), options.getDisabledProtocols());
+        assertNull(options.getEnabledCipherSuites());
 
-        assertEquals(TransportSslOptions.DEFAULT_CONTEXT_PROTOCOL, sslOptions.getContextProtocol());
-        assertNull(sslOptions.getEnabledProtocols());
-        assertArrayEquals(TransportSslOptions.DEFAULT_DISABLED_PROTOCOLS.toArray(new String[0]), sslOptions.getDisabledProtocols());
-        assertNull(sslOptions.getEnabledCipherSuites());
-
-        assertEquals(TransportSslOptions.DEFAULT_STORE_TYPE, sslOptions.getKeyStoreType());
-        assertEquals(TransportSslOptions.DEFAULT_STORE_TYPE, sslOptions.getTrustStoreType());
-        assertEquals(TransportSslOptions.DEFAULT_VERIFY_HOST, sslOptions.isVerifyHost());
-        assertNull(sslOptions.getKeyAlias());
+        assertEquals(TransportOptions.DEFAULT_STORE_TYPE, options.getKeyStoreType());
+        assertEquals(TransportOptions.DEFAULT_STORE_TYPE, options.getTrustStoreType());
+        assertEquals(TransportOptions.DEFAULT_VERIFY_HOST, options.isVerifyHost());
+        assertNull(options.getKeyAlias());
     }
 
     @Test(expected = IllegalArgumentException.class)
@@ -128,6 +132,8 @@ public class NettySslTransportFactoryTest {
             "transport.tcpKeepAlive=" + CUSTOM_TCP_KEEP_ALIVE + "&" +
             "transport.soLinger=" + CUSTOM_SO_LINGER + "&" +
             "transport.soTimeout=" + CUSTOM_SO_TIMEOUT + "&" +
+            "transport.localAddress=" + CUSTOM_LOCAL_ADDRESS + "&" +
+            "transport.localPort=" + CUSTOM_LOCAL_PORT + "&" +
             "transport.verifyHost=" + CUSTOM_VERIFY_HOST + "&" +
             "transport.storeType=" + CUSTOM_STORE_TYPE + "&" +
             "transport.trustAll=" + CUSTOM_TRUST_ALL + "&" +
@@ -158,9 +164,11 @@ public class NettySslTransportFactoryTest {
         assertEquals(CUSTOM_TCP_KEEP_ALIVE, options.isTcpKeepAlive());
         assertEquals(CUSTOM_SO_LINGER, options.getSoLinger());
         assertEquals(CUSTOM_SO_TIMEOUT, options.getSoTimeout());
+        assertEquals(CUSTOM_LOCAL_ADDRESS, options.getLocalAddress());
+        assertEquals(CUSTOM_LOCAL_PORT, options.getLocalPort());
 
-        assertTrue(options instanceof TransportSslOptions);
-        TransportSslOptions sslOptions = (TransportSslOptions) options;
+        assertTrue(options instanceof TransportOptions);
+        TransportOptions sslOptions = options;
 
         List<String> enabledProtocols = Arrays.asList(sslOptions.getEnabledProtocols());
         List<String> customProtocols = Arrays.asList(CUSTOM_ENABLED_PROTOCOLS);
@@ -194,11 +202,11 @@ public class NettySslTransportFactoryTest {
         NettySslTransportFactory factory = new NettySslTransportFactory();
         Transport transport = factory.createTransport(configuredURI);
         TransportOptions options = transport.getTransportOptions();
-        assertTrue(options instanceof TransportSslOptions);
-        TransportSslOptions sslOptions = (TransportSslOptions) options;
+        assertTrue(options instanceof TransportOptions);
+        TransportOptions sslOptions = options;
 
         assertEquals(CUSTOM_STORE_TYPE, sslOptions.getKeyStoreType());
-        assertEquals(TransportSslOptions.DEFAULT_STORE_TYPE, sslOptions.getTrustStoreType());
+        assertEquals(TransportOptions.DEFAULT_STORE_TYPE, sslOptions.getTrustStoreType());
     }
 
     @Test
@@ -210,10 +218,10 @@ public class NettySslTransportFactoryTest {
         NettySslTransportFactory factory = new NettySslTransportFactory();
         Transport transport = factory.createTransport(configuredURI);
         TransportOptions options = transport.getTransportOptions();
-        assertTrue(options instanceof TransportSslOptions);
-        TransportSslOptions sslOptions = (TransportSslOptions) options;
+        assertTrue(options instanceof TransportOptions);
+        TransportOptions sslOptions = options;
 
-        assertEquals(TransportSslOptions.DEFAULT_STORE_TYPE, sslOptions.getKeyStoreType());
+        assertEquals(TransportOptions.DEFAULT_STORE_TYPE, sslOptions.getKeyStoreType());
         assertEquals(CUSTOM_STORE_TYPE, sslOptions.getTrustStoreType());
     }
 
@@ -227,10 +235,28 @@ public class NettySslTransportFactoryTest {
         NettySslTransportFactory factory = new NettySslTransportFactory();
         Transport transport = factory.createTransport(configuredURI);
         TransportOptions options = transport.getTransportOptions();
-        assertTrue(options instanceof TransportSslOptions);
-        TransportSslOptions sslOptions = (TransportSslOptions) options;
+        assertTrue(options instanceof TransportOptions);
 
-        assertEquals(CUSTOM_STORE_TYPE_PKCS12, sslOptions.getKeyStoreType());
-        assertEquals(CUSTOM_STORE_TYPE, sslOptions.getTrustStoreType());
+        assertEquals(CUSTOM_STORE_TYPE_PKCS12, options.getKeyStoreType());
+        assertEquals(CUSTOM_STORE_TYPE, options.getTrustStoreType());
+    }
+
+    @Test
+    public void testUrlEncodedTransportValues() throws Exception {
+        URI BASE_URI = new URI("tcp://localhost:5672");
+        final String encodedLocation = URLEncoder.encode(CUSTOM_TRUST_STORE_LOCATION, "utf-8");
+        final String encodedPassword = URLEncoder.encode(CUSTOM_TRUST_STORE_PASSWORD, "utf-8");
+        URI configuredURI = new URI(BASE_URI.toString() + "?" +
+                "transport.trustStoreLocation=" + encodedLocation + "&" +
+                "transport.trustStorePassword=" + encodedPassword);
+
+        NettySslTransportFactory factory = new NettySslTransportFactory();
+        Transport transport = factory.createTransport(configuredURI);
+        TransportOptions options = transport.getTransportOptions();
+        assertTrue(options instanceof TransportOptions);
+        TransportOptions sslOptions = options;
+
+        assertEquals(CUSTOM_TRUST_STORE_LOCATION, sslOptions.getTrustStoreLocation());
+        assertEquals(CUSTOM_TRUST_STORE_PASSWORD, sslOptions.getTrustStorePassword());
     }
 }
